﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Xml.Linq;
using System.Xml.XPath;
using Infragistics.Windows.DataPresenter;

namespace LoadCustomizationsProblem
{
    public partial class MainWindow : Window
    {
        private static readonly List<Field> Fields = new List<Field>
            {
                new UnboundField {Name = "Field1", Label = "1", BindingPath = new PropertyPath("Field1", null)}, 
                new UnboundField {Name = "Field2", Label = "2", BindingPath = new PropertyPath("Field2", null)}, 
                new UnboundField {Name = "Field3", Label = "3", BindingPath = new PropertyPath("Field3", null)}, 
                new UnboundField {Name = "Field4", Label = "4", BindingPath = new PropertyPath("Field4", null)}
            };

        public MainWindow()
        {
            InitializeComponent();

            _grid.FieldLayouts.Add(new FieldLayout());


            foreach (var field in Fields)
                _grid.DefaultFieldLayout.Fields.Add(field);

            _grid.DefaultFieldLayout.RecordFilters.Add(new RecordFilter(Fields[0]));

            _grid.DataSource = GetDataSource();
        }
        private IEnumerable<Foo> GetDataSource()
        {
            var list = new List<Foo>();
            for (int i = 0; i < 1000; i++)
                list.Add(new Foo {Field1 = i.ToString()});
            return list;
        }
        private void ReplaceFields()
        {
            var customs1 = _grid.SaveCustomizations();
            var xmlLayoutData = XDocument.Parse(customs1);
            xmlLayoutData.XPathSelectElements("xamDataPresenter/fieldLayouts/fieldLayout/fields/field").ElementAt(1).Remove();
            var customs2 = xmlLayoutData.ToString();

            _grid.DefaultFieldLayout.RecordFilters.Clear();
            _grid.DefaultFieldLayout.Fields.Clear();
           
            // hack.
            // uncommenting this line will make the exception go away, but will create deadlock with large layouts.
            // _grid.Dispatcher.Invoke(new Action(() => { }));

            _grid.ClearCustomizations(CustomizationType.All);

            _grid.DefaultFieldLayout.Fields.Add(Fields[2]);
            _grid.DefaultFieldLayout.Fields.Add(Fields[0]);
            _grid.LoadCustomizations(customs2);
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            ReplaceFields();
        }
    }

    internal class Foo
    {
        public string Field1 { get; set; }
        public string Field2 { get; set; }
    }
}
